@@ -1,4 +1,4 @@
-/* $Id: glibmm.h,v 1.1.1.1 2003/01/07 16:58:23 murrayc Exp $ */
+/* $Id: glibmm.h 638 2008-03-07 20:41:32Z jjongsma $ */
 
 /* glibmm - a C++ wrapper for the GLib toolkit
  *
@@ -23,7 +23,9 @@
 #define _GLIBMM_H
 
 #include <glibmmconfig.h>
+//#include <glibmm/i18n.h> //This must be included by the application, after system headers such as <iostream>.
 #include <glibmm/arrayhandle.h>
+#include <glibmm/checksum.h>
 #include <glibmm/class.h>
 #include <glibmm/containerhandle_shared.h>
 #include <glibmm/convert.h>
@@ -36,6 +38,8 @@
 #include <glibmm/helperlist.h>
 #include <glibmm/interface.h>
 #include <glibmm/iochannel.h>
+#include <glibmm/init.h>
+#include <glibmm/keyfile.h>
 #include <glibmm/streamiochannel.h>
 #include <glibmm/listhandle.h>
 #include <glibmm/main.h>
@@ -44,12 +48,14 @@
 #include <glibmm/module.h>
 #include <glibmm/objectbase.h>
 #include <glibmm/object.h>
+#include <glibmm/optioncontext.h>
 #include <glibmm/pattern.h>
 #include <glibmm/property.h>
 #include <glibmm/propertyproxy_base.h>
 #include <glibmm/propertyproxy.h>
 #include <glibmm/quark.h>
 #include <glibmm/random.h>
+#include <glibmm/regex.h>
 #include <glibmm/refptr.h>
 #include <glibmm/shell.h>
 #include <glibmm/signalproxy_connectionnode.h>
@@ -61,6 +67,7 @@
 #include <glibmm/threadpool.h>
 #include <glibmm/timer.h>
 #include <glibmm/timeval.h>
+#include <glibmm/uriutils.h>
 #include <glibmm/ustring.h>
 #include <glibmm/value.h>
 #include <glibmm/wrap.h>
