@@ -78,15 +78,33 @@ sub error
   printf STDERR "Output.pm: $format",@_;
 }
 
+sub ifdef($$)
+{
+	my ($self, $ifdef) = @_;
+	if ($ifdef)
+	{
+		$self->append("\n#ifdef $ifdef\n");
+	}
+}
+
+sub endif($$)
+{
+	my ($self, $ifdef) = @_;
+	if ($ifdef)
+	{
+		$self->append("\n#endif // $ifdef\n");
+	}
+}
+
 ### Convert _WRAP to a virtual 
 # _VFUNC_H(signame,rettype,`<cppargs>')
 # _VFUNC_PH(gtkname,crettype,cargs and names)
 # void output_wrap_vfunc_h($filename, $line_num, $objCppfunc, $objCDefsFunc)
-sub output_wrap_vfunc_h($$$$$)
+sub output_wrap_vfunc_h($$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef) = @_;
 
-#TODO: We can probably remove _VFUNC_H from the .m4 file
+#Old code. We removed _VFUNC_H from the .m4 file
 #  my $str = sprintf("_VFUNC_H(%s,%s,\`%s\',%s)dnl\n",
 #    $$objCppfunc{name},
 #    $$objCppfunc{rettype},
@@ -101,23 +119,28 @@ sub output_wrap_vfunc_h($$$$$)
     $cppVfuncDecl .= " const";
   }
 
-  $self->append("  $cppVfuncDecl;");
+  $self->append("#ifdef GLIBMM_VFUNCS_ENABLED\n");
+  $self->ifdef($ifdef);
+  $self->append("  $cppVfuncDecl;\n");
+  $self->endif($ifdef);
+  $self->append("#endif //GLIBMM_VFUNCS_ENABLED\n");
 
   #The default callback, which will call *_vfunc, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
 
-  my $str = sprintf("_VFUNC_PH(%s,%s,\`%s\')dnl\n",
+  my $str = sprintf("_VFUNC_PH(%s,%s,\`%s\',%s)dnl\n",
     $$objCDefsFunc{name},
     $$objCDefsFunc{rettype},
     $objCDefsFunc->args_types_and_names(),
+    $ifdef
    );
   $self->append($str);
 }
 
 # _VFUNC_CC(signame,gtkname,rettype,crettype,`<cppargs>',`<cargs>')
-sub output_wrap_vfunc_cc($$$$$$)
+sub output_wrap_vfunc_cc($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $ifdef) = @_;
 
   my $cname = $$objDefsSignal{name};
 
@@ -128,7 +151,7 @@ sub output_wrap_vfunc_cc($$$$$$)
   my $refreturn = "";
   $refreturn = "refreturn" if($$objCppfunc{rettype_needs_ref});
 
-  my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s)dnl\n",
+  my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
     $$objCppfunc{name},
     $cname,
     $$objCppfunc{rettype},
@@ -136,7 +159,8 @@ sub output_wrap_vfunc_cc($$$$$$)
     $objCppfunc->args_types_and_names(),
     convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
     $objCppfunc->get_is_const(),
-    $refreturn);
+    $refreturn,
+    $ifdef);
 
   $self->append($str);
 
@@ -145,7 +169,7 @@ sub output_wrap_vfunc_cc($$$$$$)
   my $refreturn_ctype = "";
   $refreturn_ctype = "refreturn_ctype" if($$objDefsSignal{rettype_needs_ref});
 
-  my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s)dnl\n",
+  my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s,%s)dnl\n",
     $$objCppfunc{name},
     $cname,
     $$objCppfunc{rettype},
@@ -154,23 +178,25 @@ sub output_wrap_vfunc_cc($$$$$$)
     $objDefsSignal->args_names_only(),
     convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
     ${$objDefsSignal->get_param_names()}[0],
-    $refreturn_ctype);
+    $refreturn_ctype,
+    $ifdef);
 
   $self->append($str);
 }
 
 ### Convert _WRAP to a virtual
-# _SIGNAL_H(signame,rettype,`<cppargs>')
-# _SIGNAL_PH(gtkname,crettype,cargs and names)
-# void output_wrap_default_signal_handler_h($filename, $line_num, $objCppfunc, $objCDefsFunc, @args)
-sub output_wrap_default_signal_handler_h($$$$$$)
+# _SIGNAL_H(signame,rettype, ifdef, `<cppargs>')
+# _SIGNAL_PH(gtkname,crettype, ifdef, cargs and names)
+# void output_wrap_default_signal_handler_h($filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef. @args)
+sub output_wrap_default_signal_handler_h($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $bImplement) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef) = @_;
 
-  my $str = sprintf("_SIGNAL_H(%s,%s,\`%s\')dnl\n",
+  my $str = sprintf("_SIGNAL_H(%s,%s,\`%s\',%s)dnl\n",
     $$objCppfunc{name},
     $$objCppfunc{rettype},
-    $objCppfunc->args_types_and_names()
+    $objCppfunc->args_types_and_names(),
+    $ifdef
    );
   $self->append($str);
 
@@ -178,32 +204,38 @@ sub output_wrap_default_signal_handler_h($$$$$$)
   #The default callback, which will call *_impl, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
 
-  $str = sprintf("_SIGNAL_PH(%s,%s,\`%s\')dnl\n",
+  $str = sprintf("_SIGNAL_PH(%s,%s,\`%s\',%s)dnl\n",
     $$objCDefsFunc{name},
     $$objCDefsFunc{rettype},
-    $objCDefsFunc->args_types_and_names()
+    $objCDefsFunc->args_types_and_names(),
+    $ifdef
    );
   $self->append($str);
 }
 
 # _SIGNAL_CC(signame, gtkname, rettype, crettype,`<cppargs>',`<cargs>')
-sub output_wrap_default_signal_handler_cc($$$$$)
+sub output_wrap_default_signal_handler_cc($$$$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement, $bCustomCCallback, $bRefreturn, $ifdef) = @_;
   my $cname = $$objDefsSignal{name};
   # $cname = $1 if ($args[3] =~ /"(.*)"/); #TODO: What's this about?
 
   # e.g. Gtk::Button::on_clicked:
   if($bImplement eq 1)
   {
-    my $str = sprintf("_SIGNAL_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s)dnl\n",
+    my $refreturn = "";
+    $refreturn = "refreturn" if($bRefreturn eq 1);
+  
+    my $str = sprintf("_SIGNAL_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
       $$objCppfunc{name},
       $cname,
       $$objCppfunc{rettype},
       $$objDefsSignal{rettype},
       $objCppfunc->args_types_and_names(),
       convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
-      $$objCppfunc{const});
+      $$objCppfunc{const},
+      $refreturn,
+      $ifdef);
     $self->append($str);
   }
 
@@ -219,47 +251,117 @@ sub output_wrap_default_signal_handler_cc($$$$$)
 #    $arglist_names .= $arglist_names_extra;
 #  }
 
-  my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s)dnl\n",
-    $$objCppfunc{name},
-    $cname,
-    $$objCppfunc{rettype},
-    $$objDefsSignal{rettype},
-    $objDefsSignal->args_types_and_names(),
-    $objDefsSignal->args_names_only(),
-    convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
-    ${$objDefsSignal->get_param_names()}[0]);
-  $self->append($str);
+  if($bCustomCCallback ne 1)
+  {
+    my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',\`%s\',%s)dnl\n",
+      $$objCppfunc{name},
+      $cname,
+      $$objCppfunc{rettype},
+      $$objDefsSignal{rettype},
+      $objDefsSignal->args_types_and_names(),
+      $objDefsSignal->args_names_only(),
+      convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
+      ${$objDefsSignal->get_param_names()}[0],
+      $ifdef);
+    $self->append($str);
+  }
 }
 
 ### Convert _WRAP to a method
 #  _METHOD(cppname,cname,cpprettype,crettype,arglist,cargs,const)
-#  void output_wrap_meth($filename, $line_num, $objCppFunc, $objCDefsFunc, $cppMethodDecl, $documentation)
-sub output_wrap_meth($$$$$$)
+#  void output_wrap_meth($filename, $line_num, $objCppFunc, $objCDefsFunc, $cppMethodDecl, $documentation, $ifdef)
+sub output_wrap_meth($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $cppMethodDecl, $documentation) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $cppMethodDecl, $documentation, $ifdef) = @_;
   my $objDefsParser = $$self{objDefsParser};
 
+  # Allow the generated .h/.cc code to have an #ifndef around it, and add deprecation docs to the generated documentation.
+  my $deprecated = "";
+  if($$objCDefsFunc{deprecated})
+  {
+    $deprecated = "deprecated";
+  }
+
+  #Declaration:
+  if($deprecated ne "")
+  {
+    $self->append("\n_DEPRECATE_IFDEF_START");
+  }
+
   # Doxygen documentation before the method declaration:
   $self->output_wrap_meth_docs_only($filename, $line_num, $documentation);
 
-  #Declaration:
+ $self->ifdef($ifdef);
+
+  if($$objCDefsFunc{throw_any_errors})
+  {
+     $self->append("#ifdef GLIBMM_EXCEPTIONS_ENABLED\n");
+  }
+
   $self->append("  ${cppMethodDecl};");
 
+  if($$objCDefsFunc{throw_any_errors})
+  {
+     $self->append("\n#else\n");
+
+     # #Add an error argument, by searching for ) at the end and replacing it:
+     # my $declWithErrorArg = ${cppMethodDecl};
+     # $declWithErrorArg =~ s/\)$/, std::auto_ptr<Glib::Error>& error\)/g;
+
+     #Recreate the declaration, to remove the default values, which we can't have as well as an error parameter at the end: 
+     my $declWithErrorArg = $$objCppfunc{rettype} . " " . $$objCppfunc{name} . "(" . $objCppfunc->args_types_and_names() . ", std::auto_ptr<Glib::Error>& error)";
+
+     if($$objCppfunc{static})
+     {
+       $declWithErrorArg = "static " . $declWithErrorArg;
+     }
+
+     if($objCppfunc->get_is_const() eq 1)
+     {
+       if($$objCppfunc{static} ne 1) #It can't be const and static at the same time.
+       {
+         $declWithErrorArg = $declWithErrorArg . " const";
+       }
+     }
+
+     #remove any superfluous ,:
+     $declWithErrorArg =~ s/\(, /\(/g;
+
+     $self->append("  ${declWithErrorArg};");
+
+     $self->append("\n#endif //GLIBMM_EXCEPTIONS_ENABLED\n");
+  }
+  
+  $self->endif($ifdef);
+
+  
+  if($deprecated ne "")
+  {
+    $self->append("\n_DEPRECATE_IFDEF_END\n");
+  }
+
   my $refneeded = "";
   if($$objCDefsFunc{rettype_needs_ref})
   {
     $refneeded = "refreturn"
   }
+
   my $errthrow = "";
   if($$objCDefsFunc{throw_any_errors})
   {
     $errthrow = "errthrow"
   }
 
+  my $constversion = ""; #Whether it is just a const overload (so it can reuse code)
+  if($$objCDefsFunc{constversion})
+  {
+    $constversion = "constversion"
+  }
+
   #Implementation:
   my $str;
   if ($$objCppfunc{static}) {
-    $str = sprintf("_STATIC_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s)dnl\n",
+    $str = sprintf("_STATIC_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s,%s)dnl\n",
       $$objCppfunc{name},
       $$objCDefsFunc{c_name},
       $$objCppfunc{rettype},
@@ -267,9 +369,11 @@ sub output_wrap_meth($$$$$$)
       $objCppfunc->args_types_and_names(),
       convert_args_cpp_to_c($objCppfunc, $objCDefsFunc, 1, $line_num, $errthrow), #1 means it's static, so it has 'object'.
       $refneeded,
-      $errthrow);
+      $errthrow,
+      $deprecated,
+      $ifdef);
   } else {
-    $str = sprintf("_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
+    $str = sprintf("_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s,%s,%s,\`%s\',%s)dnl\n",
       $$objCppfunc{name},
       $$objCDefsFunc{c_name},
       $$objCppfunc{rettype},
@@ -278,10 +382,13 @@ sub output_wrap_meth($$$$$$)
       convert_args_cpp_to_c($objCppfunc, $objCDefsFunc, 0, $line_num, $errthrow),
       $$objCppfunc{const},
       $refneeded,
-      $errthrow);
+      $errthrow,
+      $deprecated,
+      $constversion,
+      $objCppfunc->args_names_only(),
+      $ifdef
+      );
   }
-
-
   $self->append($str);
 }
 
@@ -338,17 +445,43 @@ sub output_wrap_create($$$)
   $self->append($str)
 }
 
-# void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name)
+# void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback)
 # custom_signalproxy_name is "" when no type conversion is required - a normal templates SignalProxy will be used instead.
-sub output_wrap_sig_decl($$$$$$)
+sub output_wrap_sig_decl($$$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name) = @_;
+  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback, $ifdef, $merge_doxycomment_with_previous) = @_;
 
 # _SIGNAL_PROXY(c_signal_name, c_return_type, `<c_arg_types_and_names>',
 #               cpp_signal_name, cpp_return_type, `<cpp_arg_types>',`<c_args_to_cpp>',
 #               refdoc_comment)
 
-  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\')dnl\n",
+  my $doxycomment = $objCppfunc->get_refdoc_comment();
+
+  # If there was already a previous doxygen comment, we want to merge this
+  # one with the previous so it is one big comment. If it were two separate
+  # comments, doxygen would ignore the first one. If
+  # $merge_doxycomment_with_previous is nonzero, the first comment is
+  # already open but not yet closed.
+  if($merge_doxycomment_with_previous)
+  {
+    # Strip leading whitespace
+    $doxycomment =~ s/^\s+//;
+    # We don't have something to add, so just close the comment.
+    if($doxycomment eq "")
+    {
+      $doxycomment = "   */";
+    }
+    else
+    {
+      # Append the new comment, but remove the first three leading characters
+      # (which are /**) that mark the beginning of the comment.
+      $doxycomment = substr($doxycomment, 3);
+      $doxycomment =~ s/^\s+//;
+      $doxycomment = "   " . $doxycomment;
+    }
+  }
+
+  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\',\`%s\',%s)dnl\n",
     $signal_name,
     $$objCSignal{rettype},
     $objCSignal->args_types_and_names_without_object(),
@@ -356,7 +489,9 @@ sub output_wrap_sig_decl($$$$$$)
     $$objCppfunc{rettype},
     $objCppfunc->args_types_only(),
     convert_args_c_to_cpp($objCSignal, $objCppfunc, $line_num),
-    $objCppfunc->get_refdoc_comment()
+    $bCustomCCallback, #When this is true, it will not write the *_callback implementation for you.
+    $doxycomment,
+    $ifdef
   );
 
   $self->append($str);
@@ -463,6 +598,12 @@ sub output_wrap_property($$$$$$)
     {
       # construct-only functions can be read, but not written.
       $proxy_suffix = "_ReadOnly";
+
+      if($objProperty->get_readable() ne 1)
+      {
+        $self->output_wrap_failed($name, "attempt to wrap write-only and construct-only property.");
+        return;
+      }
     }
     elsif($objProperty->get_readable() ne 1)
     {
@@ -472,24 +613,30 @@ sub output_wrap_property($$$$$$)
     {
        $proxy_suffix = "_ReadOnly";
     }
-    
-    $name =~ s/-/_/g;
 
-    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s)dnl\n",
+    # Convert - to _ so we can use it in C++ method and variable names:
+    my $name_underscored = $name;
+    $name_underscored =~ s/-/_/g;
+
+    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
       $name,
+      $name_underscored,
       $cpp_type,
-      $proxy_suffix
+      $proxy_suffix,
+      $objProperty->get_docs()
     );
     $self->append($str);
     $self->append("\n");
 
-    # If the property is not already read-only, then add a second const accessor for a read-only propertyproxy:
-    if($proxy_suffix ne "_ReadOnly")
+    # If the property is not already read-only, and the property can be read, then add a second const accessor for a read-only propertyproxy:
+    if( ($proxy_suffix ne "_ReadOnly") && ($objProperty->get_readable()) )
     {
-      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s)dnl\n",
+      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
         $name,
+        $name_underscored,
         $cpp_type,
-        "_ReadOnly"
+        "_ReadOnly",
+        $objProperty->get_docs()
       );
       $self->append($str);
     }
@@ -608,7 +755,7 @@ sub convert_args_cpp_to_c($$$$;$)
        ${$c_param_types}[-1] eq "GError**" )
   {
     $num_cpp_args++;
-    $cpp_param_names = [@{$cpp_param_names},"error"];
+    $cpp_param_names = [@{$cpp_param_names},"gerror"];
     $cpp_param_types = [@{$cpp_param_types},"GError*&"];
   }
 
@@ -799,5 +946,15 @@ sub output_wrap_corba_method($$$$)
   $self->append($str);
 }
 
+sub output_implements_interface($$)
+{
+  my ($self, $interface, $ifdef) = @_;
+
+  my $str = sprintf("_IMPLEMENTS_INTERFACE_CC(%s, %s)dnl\n",
+  	$interface,
+  	$ifdef);
+
+  $self->append($str);
+}
 
 1; # indicate proper module load.
